@@ -1,3 +1,5 @@
+import warnings
+
 import pytest
 
 pd = pytest.importorskip("pandas")
@@ -91,7 +93,8 @@ def test_ufunc(pandas_input, ufunc):
     dask_type = dask_input.__class__
 
     # applying Dask ufunc doesn't trigger computation
-    with pytest.warns(None):
+    with warnings.catch_warnings():
+        warnings.simplefilter("ignore", RuntimeWarning)
         # Some cause warnings (arcsine)
         assert isinstance(dafunc(dask_input), dask_type)
         assert_eq(dafunc(dask_input), npfunc(pandas_input))
@@ -113,7 +116,8 @@ def test_ufunc(pandas_input, ufunc):
     if ufunc in ("logical_not", "signbit", "isnan", "isinf", "isfinite"):
         return
 
-    with pytest.warns(None):
+    with warnings.catch_warnings():
+        warnings.simplefilter("ignore", RuntimeWarning)
         assert isinstance(dafunc(dask_input.index), dd.Index)
         assert_eq(
             dafunc(dask_input.index),
@@ -134,7 +138,8 @@ def test_ufunc(pandas_input, ufunc):
         )
 
     # applying Dask ufunc to normal Series triggers computation
-    with pytest.warns(None):
+    with warnings.catch_warnings():
+        warnings.simplefilter("ignore", RuntimeWarning)
         # some (da.log) cause warnings
         assert isinstance(dafunc(pandas_input.index), pd.Index)
         assert_eq(dafunc(pandas_input), npfunc(pandas_input))
@@ -329,12 +334,14 @@ def test_frame_ufunc_out(ufunc):
     ddf_out_np = dd.from_pandas(df_out, 3)
     ddf_out_da = dd.from_pandas(df_out, 3)
 
-    with pytest.warns(None):
+    with warnings.catch_warnings():
+        warnings.simplefilter("ignore", RuntimeWarning)
         npfunc(ddf, out=ddf_out_np)
         dafunc(ddf, out=ddf_out_da)
         assert_eq(ddf_out_np, ddf_out_da)
 
-    with pytest.warns(None):
+    with warnings.catch_warnings():
+        warnings.simplefilter("ignore", RuntimeWarning)
         expected = pd.DataFrame(npfunc(input_matrix), columns=["A", "B"])
         assert_eq(ddf_out_np, expected)
 
@@ -499,7 +506,9 @@ def test_ufunc_with_reduction(redfunc, ufunc, pandas):
         # (instead of being applied on 2D ndarray that was converted to float)
         pytest.xfail("'prod' overflowing with integer columns in pandas 1.2.0")
 
-    with pytest.warns(None):
+    with warnings.catch_warnings():
+        warnings.simplefilter("ignore", RuntimeWarning)
+        warnings.simplefilter("ignore", FutureWarning)
         assert isinstance(np_redfunc(dask), (dd.DataFrame, dd.Series, dd.core.Scalar))
         assert_eq(np_redfunc(np_ufunc(dask)), np_redfunc(np_ufunc(pandas)))
 
